@@ -1,12 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
 from trytond.session import Session
 from psycopg2.pool import ThreadedConnectionPool
 from psycopg2.extensions import cursor as PsycopgCursor
-from psycopg2.extensions import ISOLATION_LEVEL_SERIALIZABLE
+from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
 from psycopg2.extensions import register_type, register_adapter
 from psycopg2.extensions import UNICODE, AsIs
@@ -27,6 +26,7 @@ from trytond.protocols.datatype import Float
 
 RE_FROM = re.compile('.* from "?([a-zA-Z_0-9]+)"?.*$')
 RE_INTO = re.compile('.* into "?([a-zA-Z_0-9]+)"?.*$')
+RE_VERSION = re.compile(r'\S+ (\d+)\.(\d+)')
 
 
 class Database(DatabaseInterface):
@@ -35,6 +35,7 @@ class Database(DatabaseInterface):
     _connpool = None
     _list_cache = None
     _list_cache_timestamp = None
+    _version_cache = {}
 
     def __new__(cls, database_name='template1'):
         if database_name in cls._databases:
@@ -62,15 +63,19 @@ class Database(DatabaseInterface):
         self._connpool = ThreadedConnectionPool(minconn, maxconn, dsn)
         return self
 
-    def cursor(self, autocommit=False):
+    def cursor(self, autocommit=False, readonly=False):
         if self._connpool is None:
             self.connect()
         conn = self._connpool.getconn()
         if autocommit:
             conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
         else:
-            conn.set_isolation_level(ISOLATION_LEVEL_SERIALIZABLE)
-        return Cursor(self._connpool, conn, self.database_name)
+            conn.set_isolation_level(ISOLATION_LEVEL_REPEATABLE_READ)
+        cursor = Cursor(self._connpool, conn, self)
+        # TODO change for set_session
+        if readonly:
+            cursor.execute('SET TRANSACTION READ ONLY')
+        return cursor
 
     def close(self):
         if self._connpool is None:
@@ -87,6 +92,14 @@ class Database(DatabaseInterface):
         cursor.execute('DROP DATABASE "' + database_name + '"')
         Database._list_cache = None
 
+    def get_version(self, cursor):
+        if self.database_name not in self._version_cache:
+            cursor.execute('SELECT version()')
+            version, = cursor.fetchone()
+            self._version_cache[self.database_name] = tuple(map(int,
+                RE_VERSION.search(version).groups()))
+        return self._version_cache[self.database_name]
+
     @staticmethod
     def dump(database_name):
         from trytond.tools import exec_pg_command_pipe
@@ -271,12 +284,11 @@ class _Cursor(PsycopgCursor):
 
 class Cursor(CursorInterface):
 
-    def __init__(self, connpool, conn, database_name):
+    def __init__(self, connpool, conn, database):
         super(Cursor, self).__init__()
         self._connpool = connpool
         self._conn = conn
-        self.database_name = database_name
-        self.dbname = self.database_name #XXX to remove
+        self._database = database
         self.cursor = conn.cursor(cursor_factory=_Cursor)
         self.commit()
         self.sql_from_log = {}
@@ -286,6 +298,15 @@ class Cursor(CursorInterface):
             'into': 0,
         }
 
+    @property
+    def database_name(self):
+        return self._database.database_name
+
+    # TODO to remove
+    @property
+    def dbname(self):
+        return self.database_name
+
     def __getattr__(self, name):
         return getattr(self.cursor, name)
 
@@ -402,6 +423,10 @@ class Cursor(CursorInterface):
             select += ' OFFSET %d' % offset
         return select
 
+    def has_returning(self):
+        # RETURNING clause is available since PostgreSQL 8.2
+        return self._database.get_version(self) >= (8, 2)
+
 register_type(UNICODE)
 if PYDATE:
     register_type(PYDATE)
