@@ -1,6 +1,5 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import base64
 import os
 from trytond.model import ModelView, ModelSQL, ModelStorage, fields
@@ -9,6 +8,8 @@ from trytond.backend import TableHandler
 from trytond.pyson import PYSONEncoder, CONTEXT, PYSON
 from trytond.transaction import Transaction
 from trytond.cache import Cache
+from trytond.pool import Pool
+from trytond.exceptions import UserError
 
 
 class Action(ModelSQL, ModelView):
@@ -20,7 +21,7 @@ class Action(ModelSQL, ModelView):
     usage = fields.Char('Usage')
     keywords = fields.One2Many('ir.action.keyword', 'action',
             'Keywords')
-    groups = fields.Many2Many('ir.action-res.group', 'action_id', 'gid',
+    groups = fields.Many2Many('ir.action-res.group', 'action', 'group',
             'Groups')
     icon = fields.Many2One('ir.ui.icon', 'Icon')
     active = fields.Boolean('Active', select=2)
@@ -38,6 +39,7 @@ class Action(ModelSQL, ModelView):
         return True
 
     def get_action_id(self, action_id):
+        pool = Pool()
         with Transaction().set_context(active_test=False):
             if self.search([
                 ('id', '=', action_id),
@@ -49,7 +51,7 @@ class Action(ModelSQL, ModelView):
                     'ir.action.wizard',
                     'ir.action.url',
                     ):
-                action_obj = self.pool.get(action_type)
+                action_obj = pool.get(action_type)
                 action_id2 = action_obj.search([
                     ('id', '=', action_id),
                     ])
@@ -88,7 +90,8 @@ class ActionKeyword(ModelSQL, ModelView):
         })
 
     def check_wizard_model(self, ids):
-        action_wizard_obj = self.pool.get('ir.action.wizard')
+        pool = Pool()
+        action_wizard_obj = pool.get('ir.action.wizard')
         for action_keyword in self.browse(ids):
             if action_keyword.action.type == 'ir.action.wizard':
                 action_wizard_id = action_wizard_obj.search([
@@ -103,13 +106,15 @@ class ActionKeyword(ModelSQL, ModelView):
 
     def _convert_vals(self, vals):
         vals = vals.copy()
-        action_obj = self.pool.get('ir.action')
+        pool = Pool()
+        action_obj = pool.get('ir.action')
         if 'action' in vals:
             vals['action'] = action_obj.get_action_id(vals['action'])
         return vals
 
     def models_get(self):
-        model_obj = self.pool.get('ir.model')
+        pool = Pool()
+        model_obj = pool.get('ir.model')
         model_ids = model_obj.search([])
         res = []
         for model in model_obj.browse(model_ids):
@@ -117,36 +122,39 @@ class ActionKeyword(ModelSQL, ModelView):
         return res
 
     def delete(self, ids):
+        pool = Pool()
         if isinstance(ids, (int, long)):
             ids = [ids]
         for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0]
+                pool.get(keyword.model.split(',')[0]
                         ).fields_view_get.reset()
             except Exception:
                 pass
         return super(ActionKeyword, self).delete(ids)
 
     def create(self, vals):
+        pool = Pool()
         vals = self._convert_vals(vals)
         if 'model' in vals:
             # Restart the cache view
             try:
-                self.pool.get(vals['model'].split(',')[0]
+                pool.get(vals['model'].split(',')[0]
                         ).fields_view_get.reset()
             except Exception:
                 pass
         return super(ActionKeyword, self).create(vals)
 
     def write(self, ids, vals):
+        pool = Pool()
         vals = self._convert_vals(vals)
         if isinstance(ids, (int, long)):
             ids = [ids]
         for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0]
+                pool.get(keyword.model.split(',')[0]
                         ).fields_view_get.reset()
             except Exception:
                 pass
@@ -154,30 +162,32 @@ class ActionKeyword(ModelSQL, ModelView):
         for keyword in self.browse(ids):
             # Restart the cache view
             try:
-                self.pool.get(keyword.model.split(',')[0]
+                pool.get(keyword.model.split(',')[0]
                         ).fields_view_get.reset()
             except Exception:
                 pass
         return res
 
     def get_keyword(self, keyword, value):
+        pool = Pool()
         res = []
         model, model_id = value
 
         action_keyword_ids = []
-        if model_id:
+        if model_id >= 0:
             action_keyword_ids = self.search([
                 ('keyword', '=', keyword),
                 ('model', '=', model + ',' + str(model_id)),
                 ])
         action_keyword_ids.extend(self.search([
             ('keyword', '=', keyword),
-            ('model', '=', model + ',0'),
+            ('model', '=', model + ',-1'),
             ]))
-        for action_keyword in self.browse(action_keyword_ids):
+        for action_keyword_id in action_keyword_ids:
+            action_keyword = self.browse(action_keyword_id)
             try:
-                action_obj = self.pool.get(action_keyword.action.type)
-            except Exception:
+                action_obj = pool.get(action_keyword.action.type)
+            except UserError:
                 continue
             action_id = action_obj.search([
                 ('action', '=', action_keyword.action.id),
@@ -187,7 +197,7 @@ class ActionKeyword(ModelSQL, ModelView):
                     + action_obj._inherit_fields.keys())
                 columns.add('icon.rec_name')
                 if action_keyword.action.type == 'ir.action.report':
-                    to_remove = ('report_content_data', 'report_content',
+                    to_remove = ('report_content_custom', 'report_content',
                         'style_content')
                 elif action_keyword.action.type == 'ir.action.act_window':
                     to_remove = ('domain', 'context', 'search_value')
@@ -208,7 +218,7 @@ class ActionReport(ModelSQL, ModelView):
     model = fields.Char('Model')
     report_name = fields.Char('Internal Name', required=True)
     report = fields.Char('Path')
-    report_content_data = fields.Binary('Content')
+    report_content_custom = fields.Binary('Content')
     report_content = fields.Function(fields.Binary('Content'),
             'get_report_content', setter='set_report_content')
     action = fields.Many2One('ir.action', 'Action', required=True,
@@ -218,10 +228,77 @@ class ActionReport(ModelSQL, ModelView):
     style_content = fields.Function(fields.Binary('Style'),
             'get_style_content')
     direct_print = fields.Boolean('Direct Print')
-    extension = fields.Selection(
-        [('odt', 'ODT Document'),
-         ('pdf', 'PDF Document'),],
-        string='Extension', required=True)
+    template_extension = fields.Selection([
+            ('odt', 'OpenDocument Text'),
+            ('odp', 'OpenDocument Presentation'),
+            ('ods', 'OpenDocument Spreadsheet'),
+            ('odg', 'OpenDocument Graphics'),
+            ], string='Template Extension', required=True,
+        translate=False)
+    extension = fields.Selection([
+            ('', ''),
+            ('bib', 'BibTex'),
+            ('bmp', 'Windows Bitmap'),
+            ('csv', 'Text CSV'),
+            ('dbf', 'dBase'),
+            ('dif', 'Data Interchange Format'),
+            ('doc', 'Microsoft Word 97/2000/XP'),
+            ('doc6', 'Microsoft Word 6.0'),
+            ('doc95', 'Microsoft Word 95'),
+            ('docbook', 'DocBook'),
+            ('emf', 'Enhanced Metafile'),
+            ('eps', 'Encapsulated PostScript'),
+            ('gif', 'Graphics Interchange Format'),
+            ('html', 'HTML Document'),
+            ('jpg', 'Joint Photographic Experts Group'),
+            ('met', 'OS/2 Metafile'),
+            ('ooxml', 'Microsoft Office Open XML'),
+            ('pbm', 'Portable Bitmap'),
+            ('pct', 'Mac Pict'),
+            ('pdb', 'AportisDoc (Palm)'),
+            ('pdf', 'Portable Document Format'),
+            ('pgm', 'Portable Graymap'),
+            ('png', 'Portable Network Graphic'),
+            ('ppm', 'Portable Pixelmap'),
+            ('ppt', 'Microsoft PowerPoint 97/2000/XP'),
+            ('psw', 'Pocket Word'),
+            ('pwp', 'PlaceWare'),
+            ('pxl', 'Pocket Excel'),
+            ('ras', 'Sun Raster Image'),
+            ('rtf', 'Rich Text Format'),
+            ('latex', 'LaTeX 2e'),
+            ('sda', 'StarDraw 5.0 (OpenOffice.org Impress)'),
+            ('sdc', 'StarCalc 5.0'),
+            ('sdc4', 'StarCalc 4.0'),
+            ('sdc3', 'StarCalc 3.0'),
+            ('sdd', 'StarImpress 5.0'),
+            ('sdd3', 'StarDraw 3.0 (OpenOffice.org Impress)'),
+            ('sdd4', 'StarImpress 4.0'),
+            ('sdw', 'StarWriter 5.0'),
+            ('sdw4', 'StarWriter 4.0'),
+            ('sdw3', 'StarWriter 3.0'),
+            ('slk', 'SYLK'),
+            ('svg', 'Scalable Vector Graphics'),
+            ('svm', 'StarView Metafile'),
+            ('swf', 'Macromedia Flash (SWF)'),
+            ('sxc', 'OpenOffice.org 1.0 Spreadsheet'),
+            ('sxi', 'OpenOffice.org 1.0 Presentation'),
+            ('sxd', 'OpenOffice.org 1.0 Drawing'),
+            ('sxd3', 'StarDraw 3.0'),
+            ('sxd5', 'StarDraw 5.0'),
+            ('sxw', 'Open Office.org 1.0 Text Document'),
+            ('text', 'Text Encoded'),
+            ('tiff', 'Tagged Image File Format'),
+            ('txt', 'Plain Text'),
+            ('wmf', 'Windows Metafile'),
+            ('xhtml', 'XHTML Document'),
+            ('xls', 'Microsoft Excel 97/2000/XP'),
+            ('xls5', 'Microsoft Excel 5.0'),
+            ('xls95', 'Microsoft Excel 95'),
+            ('xpm', 'X PixMap'),
+            ], translate=False,
+        string='Extension', help='Leave empty for the same as template, '
+        'see unoconv documentation for compatible format')
     module = fields.Char('Module', readonly=True, select=1)
     email = fields.Char('Email')
     pyson_email = fields.Function(fields.Char('PySON Email'), 'get_pyson')
@@ -261,6 +338,26 @@ class ActionReport(ModelSQL, ModelView):
             TableHandler.dropTable(cursor, 'ir.action.report.outputformat',
                       'ir_action_report_outputformat')
 
+        # Migrate from 2.0 remove required on extension
+        table.not_null_action('extension', action='remove')
+        cursor.execute('UPDATE "' + self._table + '" '
+            'SET extension = %s '
+            'WHERE extension = %s', ('', 'odt'))
+
+        # Migration from 2.0 report_content_data renamed into
+        # report_content_custom to remove base64 encoding
+        if (table.column_exist('report_content_data')
+                and table.column_exist('report_content_custom')):
+            cursor.execute('SELECT id, report_content_data '
+                'FROM "' + self._table + '"')
+            for report_id, content in cursor.fetchall():
+                if content:
+                    content = base64.decodestring(str(content))
+                    cursor.execute('UPDATE "' + self._table + '" '
+                        'SET report_content_custom = %s '
+                        'WHERE id = %s', (content, report_id))
+            table.drop_column('report_content_data')
+
     def default_type(self):
         return 'ir.action.report'
 
@@ -270,39 +367,54 @@ class ActionReport(ModelSQL, ModelView):
     def default_direct_print(self):
         return False
 
-    def default_extension(self):
+    def default_template_extension(self):
         return 'odt'
 
+    def default_extension(self):
+        return ''
+
     def default_module(self):
         return Transaction().context.get('module') or ''
 
     def get_report_content(self, ids, name):
         res = {}
+        converter = buffer
+        default = False
+        format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
+        if format_ == 'size':
+            converter = len
+            default = 0
         for report in self.browse(ids):
-            data = report[name + '_data']
+            data = report[name + '_custom']
             if not data and report[name[:-8]]:
                 try:
-                    with file_open( report[name[:-8]].replace('/', os.sep),
+                    with file_open(report[name[:-8]].replace('/', os.sep),
                             mode='rb') as fp:
-                        data = base64.encodestring(fp.read())
+                        data = fp.read()
                 except Exception:
                     data = False
-            res[report.id] = data
+            res[report.id] = converter(data) if data else default
         return res
 
     def set_report_content(self, ids, name, value):
-        self.write(ids, {'%s_data' % name: value})
+        self.write(ids, {'%s_custom' % name: value})
 
     def get_style_content(self, ids, name):
         res = {}
+        converter = buffer
+        default = False
+        format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
+        if format_ == 'size':
+            converter = len
+            default = 0
         for report in self.browse(ids):
             try:
                 with file_open( report.style.replace('/', os.sep),
                         mode='rb') as fp:
-                    data = base64.encodestring(fp.read())
+                    data = fp.read()
             except Exception:
                 data = False
-            res[report.id] = data
+            res[report.id] = converter(data) if data else default
         return res
 
     def get_pyson(self, ids, name):
@@ -369,7 +481,8 @@ class ActionReport(ModelSQL, ModelView):
         return super(ActionReport, self).write(ids, vals)
 
     def delete(self, ids):
-        action_obj = self.pool.get('ir.action')
+        pool = Pool()
+        action_obj = pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -415,15 +528,23 @@ class ActionActWindow(ModelSQL, ModelView):
             ('check_views', 'invalid_views'),
             ('check_domain', 'invalid_domain'),
             ('check_context', 'invalid_context'),
-            ('check_search_value', 'invalid_search_value'),
         ]
         self._error_messages.update({
             'invalid_views': 'Invalid views!',
-            'invalid_domain': 'Invalid domain!',
+            'invalid_domain': 'Invalid domain or search criteria!',
             'invalid_context': 'Invalid context!',
-            'invalid_search_value': 'Invalid search criteria!',
         })
 
+    def init(self, module_name):
+        cursor = Transaction().cursor
+        super(ActionActWindow, self).init(module_name)
+
+        # Migration from 2.0: new search_value format
+        cursor.execute('UPDATE "%s" '
+            'SET search_value = %%s '
+            'WHERE search_value = %%s' % self._table,
+            ('[]', '{}'))
+
     def default_type(self):
         return 'ir.action.act_window'
 
@@ -443,7 +564,7 @@ class ActionActWindow(ModelSQL, ModelView):
         return True
 
     def default_search_value(self):
-        return '{}'
+        return '[]'
 
     def check_views(self, ids):
         "Check views"
@@ -465,11 +586,13 @@ class ActionActWindow(ModelSQL, ModelView):
         return True
 
     def check_domain(self, ids):
-        "Check domain"
+        "Check domain and search_value"
         for action in self.browse(ids):
-            if action.domain:
+            for domain in (action.domain, action.search_value):
+                if not domain:
+                    continue
                 try:
-                    value = safe_eval(action.domain, CONTEXT)
+                    value = safe_eval(domain, CONTEXT)
                 except Exception:
                     return False
                 if isinstance(value, PYSON):
@@ -504,21 +627,6 @@ class ActionActWindow(ModelSQL, ModelView):
                         return False
         return True
 
-    def check_search_value(self, ids):
-        "Check search_value"
-        for action in self.browse(ids):
-            if action.search_value:
-                try:
-                    value = safe_eval(action.search_value, CONTEXT)
-                except Exception:
-                    return False
-                if isinstance(value, PYSON):
-                    if not value.types() == set([dict]):
-                        return False
-                elif not isinstance(value, dict):
-                    return False
-        return True
-
     def get_views(self, ids, name):
         res = {}
         for act in self.browse(ids):
@@ -563,7 +671,8 @@ class ActionActWindow(ModelSQL, ModelView):
         return new_id
 
     def delete(self, ids):
-        action_obj = self.pool.get('ir.action')
+        pool = Pool()
+        action_obj = pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -639,7 +748,8 @@ class ActionWizard(ModelSQL, ModelView):
         return new_id
 
     def delete(self, ids):
-        action_obj = self.pool.get('ir.action')
+        pool = Pool()
+        action_obj = pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -652,89 +762,6 @@ class ActionWizard(ModelSQL, ModelView):
 ActionWizard()
 
 
-class ActionWizardSize(ModelSQL, ModelView):
-    "Action Wizard Size"
-    _name = 'ir.action.wizard_size'
-    _description = __doc__
-    wizard = fields.Char('Wizard', required=True, select=1)
-    model = fields.Char('Model', required=True, select=1)
-    user = fields.Many2One('res.user', 'User', required=True,
-            ondelete='CASCADE', select=1)
-    width = fields.Integer('Width')
-    height = fields.Integer('Height')
-
-    def __init__(self):
-        super(ActionWizardSize, self).__init__()
-        self._rpc.update({
-            'set_size': True,
-        })
-
-    def create(self, vals):
-        res = super(ActionWizardSize, self).create(vals)
-        # Restart the cache for get_size
-        self.get_size.reset()
-        return res
-
-    def write(self, ids, vals):
-        res = super(ActionWizardSize, self).write(ids, vals)
-        # Restart the cache for get_size
-        self.get_size.reset()
-        return res
-
-    def delete(self, ids):
-        res = super(ActionWizardSize, self).delete(ids)
-        # Restart the cache for get_size
-        self.get_size.reset()
-        return res
-
-    def set_size(self, wizard, model, width, height):
-        '''
-        Set size for wizard dialog.
-        :param wizard: the wizard name
-        :param model: the model name
-        :param width: the width
-        :param height: the height
-        '''
-        ids = self.search([
-            ('user', '=', Transaction().user),
-            ('wizard', '=', wizard),
-            ('model', '=', model),
-            ])
-        if ids:
-            self.write(ids, {
-                'width': width,
-                'height': height,
-                })
-        else:
-            self.create({
-                'wizard': wizard,
-                'model': model,
-                'user': Transaction().user,
-                'width': width,
-                'height': height,
-                })
-
-    @Cache('ir.action.wizard_size.get_size')
-    def get_size(self, wizard, model):
-        '''
-        Get size for wizard dialog.
-        :param wizard: the wizard name
-        :param model: the model name
-        :return: (width, height)
-        '''
-        ids = self.search([
-            ('user', '=', Transaction().user),
-            ('wizard', '=', wizard),
-            ('model', '=', model),
-            ], limit=1)
-        if ids:
-            wizard_size = self.browse(ids[0])
-            return wizard_size.width, wizard_size.height
-        return (0, 0)
-
-ActionWizardSize()
-
-
 class ActionURL(ModelSQL, ModelView):
     "Action URL"
     _name = 'ir.action.url'
@@ -773,7 +800,8 @@ class ActionURL(ModelSQL, ModelView):
         return new_id
 
     def delete(self, ids):
-        action_obj = self.pool.get('ir.action')
+        pool = Pool()
+        action_obj = pool.get('ir.action')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
