@@ -3,6 +3,7 @@
 from itertools import chain
 from trytond.model.fields.field import Field
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 class Many2Many(Field):
@@ -13,7 +14,7 @@ class Many2Many(Field):
 
     def __init__(self, relation_name, origin, target, string='', order=None,
             datetime_field=None, help='', required=False, readonly=False,
-            domain=None, states=None, change_default=False, select=0,
+            domain=None, states=None, change_default=False,
             on_change=None, on_change_with=None, depends=None,
             order_field=None, context=None, loading='lazy'):
         '''
@@ -34,7 +35,7 @@ class Many2Many(Field):
                 depends = [datetime_field]
         super(Many2Many, self).__init__(string=string, help=help,
                 required=required, readonly=readonly, domain=domain,
-                states=states, change_default=change_default, select=select,
+                states=states, change_default=change_default,
                 on_change=on_change, on_change_with=on_change_with,
                 depends=depends, order_field=order_field, context=context,
                 loading=loading)
@@ -63,7 +64,12 @@ class Many2Many(Field):
         for i in ids:
             res[i] = []
 
-        relation_obj = model.pool.get(self.relation_name)
+        if self.order is None:
+            order = [(self.target, 'ASC')]
+        else:
+            order = self.order
+
+        relation_obj = Pool().get(self.relation_name)
 
         relation_ids = []
         for i in range(0, len(ids), Transaction().cursor.IN_MAX):
@@ -71,12 +77,11 @@ class Many2Many(Field):
             relation_ids.append(relation_obj.search([
                 (self.origin, 'in', sub_ids),
                 (self.target + '.id', '!=', False),
-                ], order=self.order))
+                ], order=order))
         relation_ids = list(chain(*relation_ids))
 
-        for relation in relation_obj.read(relation_ids,
-                [self.origin, self.target]):
-            res[relation[self.origin]].append(relation[self.target])
+        for relation in relation_obj.browse(relation_ids):
+            res[relation[self.origin].id].append(relation[self.target].id)
         return res
 
     def set(self, ids, model, name, values):
@@ -97,8 +102,8 @@ class Many2Many(Field):
         '''
         if not values:
             return
-        relation_obj = model.pool.get(self.relation_name)
-        target_obj = self.get_target(model.pool)
+        relation_obj = Pool().get(self.relation_name)
+        target_obj = self.get_target()
         for act in values:
             if act[0] == 'create':
                 for record_id in ids:
@@ -173,20 +178,19 @@ class Many2Many(Field):
             else:
                 raise Exception('Bad arguments')
 
-    def get_target(self, pool):
+    def get_target(self):
         '''
         Return the target model.
 
-        :param pool: The pool
         :return: A Model
         '''
-        relation_obj = pool.get(self.relation_name)
+        relation_obj = Pool().get(self.relation_name)
         if not self.target:
             return relation_obj
         if self.target in relation_obj._columns:
-            target_obj = pool.get(
+            target_obj = Pool().get(
                     relation_obj._columns[self.target].model_name)
         else:
-            target_obj = pool.get(
+            target_obj = Pool().get(
                     relation_obj._inherit_fields[self.target][2].model_name)
         return target_obj
