@@ -4,7 +4,9 @@ from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
 from trytond.protocols.datatype import Float
-from trytond.protocols.common import daemon
+from trytond.protocols.common import daemon, GZipRequestHandlerMixin
+from trytond.exceptions import UserError, UserWarning, NotLogged, \
+    ConcurrencyException
 from trytond import security
 import SimpleXMLRPCServer
 import SocketServer
@@ -13,8 +15,6 @@ import traceback
 import socket
 import sys
 import os
-import gzip
-import StringIO
 import base64
 import datetime
 from types import DictType
@@ -27,6 +27,12 @@ def dump_decimal(self, value, write):
     write(str(value))
     write("</double></value>\n")
 
+def dump_buffer(self, value, write):
+    self.write = write
+    value = xmlrpclib.Binary(value)
+    value.encode(self)
+    del self.write
+
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpclib.Marshaller.dispatch[Float] = dump_decimal
 xmlrpclib.Marshaller.dispatch[type(None)] = \
@@ -34,6 +40,7 @@ xmlrpclib.Marshaller.dispatch[type(None)] = \
 xmlrpclib.Marshaller.dispatch[datetime.date] = \
         lambda self, value, write: self.dump_datetime(
                 datetime.datetime.combine(value, datetime.time()), write)
+xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
 
 def dump_struct(self, value, write, escape=xmlrpclib.escape):
     converted_value = {}
@@ -83,6 +90,13 @@ class GenericXMLRPCRequestHandler:
                     }
                 return dispatch(host, port, 'XML-RPC', database_name, user,
                         session, object_type, object_name, method, *params)
+            except (NotLogged, ConcurrencyException), exception:
+                raise xmlrpclib.Fault(exception.code,
+                    '\n'.join(exception.args))
+            except (UserError, UserWarning), exception:
+                error, description = exception.args
+                raise xmlrpclib.Fault(exception.code,
+                    '\n'.join((error,) + description))
             except Exception:
                 tb_s = ''
                 for line in traceback.format_exception(*sys.exc_info()):
@@ -97,13 +111,15 @@ class GenericXMLRPCRequestHandler:
                     import pdb
                     traceb = sys.exc_info()[2]
                     pdb.post_mortem(traceb)
-                raise xmlrpclib.Fault(1, str(sys.exc_value) + '\n' + tb_s)
+                raise xmlrpclib.Fault(255, str(sys.exc_value) + '\n' + tb_s)
         finally:
             security.logout(database_name, user, session)
 
 
-class SimpleXMLRPCRequestHandler(GenericXMLRPCRequestHandler,
+class SimpleXMLRPCRequestHandler(GZipRequestHandlerMixin,
+        GenericXMLRPCRequestHandler,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
+    protocol_version = "HTTP/1.1"
     rpc_paths = None
     encode_threshold = 1400 # common MTU
 
@@ -127,69 +143,6 @@ class SimpleXMLRPCRequestHandler(GenericXMLRPCRequestHandler,
         self.send_header("WWW-Authenticate", 'Basic realm="Tryton"')
         return False
 
-    # Copy from SimpleXMLRPCServer.py with gzip encoding added
-    def do_POST(self):
-        """Handles the HTTP POST request.
-
-        Attempts to interpret all HTTP POST requests as XML-RPC calls,
-        which are forwarded to the server's _dispatch method for handling.
-        """
-
-        # Check that the path is legal
-        if not self.is_rpc_path_valid():
-            self.report_404()
-            return
-
-        try:
-            # Get arguments by reading body of request.
-            # We read this in chunks to avoid straining
-            # socket.read(); around the 10 or 15Mb mark, some platforms
-            # begin to have problems (bug #792570).
-            max_chunk_size = 10*1024*1024
-            size_remaining = int(self.headers["content-length"])
-            L = []
-            while size_remaining:
-                chunk_size = min(size_remaining, max_chunk_size)
-                L.append(self.rfile.read(chunk_size))
-                size_remaining -= len(L[-1])
-            data = ''.join(L)
-
-            # In previous versions of SimpleXMLRPCServer, _dispatch
-            # could be overridden in this class, instead of in
-            # SimpleXMLRPCDispatcher. To maintain backwards compatibility,
-            # check to see if a subclass implements _dispatch and dispatch
-            # using that method if present.
-            response = self.server._marshaled_dispatch(
-                    data, getattr(self, '_dispatch', None)
-                )
-        except Exception: # This should only happen if the module is buggy
-            # internal error, report as HTTP server error
-            self.send_response(500)
-            self.end_headers()
-        else:
-            # got a valid XML RPC response
-            self.send_response(200)
-            self.send_header("Content-type", "text/xml")
-
-            # Handle gzip encoding
-            if 'gzip' in self.headers.get('Accept-Encoding', '').split(',') \
-                    and len(response) > self.encode_threshold:
-                buffer = StringIO.StringIO()
-                output = gzip.GzipFile(mode='wb', fileobj=buffer)
-                output.write(response)
-                output.close()
-                buffer.seek(0)
-                response = buffer.getvalue()
-                self.send_header('Content-Encoding', 'gzip')
-
-            self.send_header("Content-length", str(len(response)))
-            self.end_headers()
-            self.wfile.write(response)
-
-            # shut down the connection
-            self.wfile.flush()
-            self.connection.shutdown(1)
-
 
 class SecureXMLRPCRequestHandler(SimpleXMLRPCRequestHandler):
 
@@ -202,10 +155,9 @@ class SecureXMLRPCRequestHandler(SimpleXMLRPCRequestHandler):
 class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn,
         SimpleXMLRPCServer.SimpleXMLRPCServer):
     timeout = 1
+    daemon_threads = True
 
     def server_bind(self):
-        # Python < 2.6 doesn't handle self.timeout
-        self.socket.settimeout(1)
         self.socket.setsockopt(socket.SOL_SOCKET,
                 socket.SO_REUSEADDR, 1)
         SimpleXMLRPCServer.SimpleXMLRPCServer.server_bind(self)
