@@ -5,8 +5,15 @@
 
 import unittest
 import datetime
+import socket
+import urllib
+
+from trytond.config import CONFIG
 from trytond.tools import reduce_ids, safe_eval, datetime_strftime, \
         reduce_domain
+from trytond.tests.test_tryton import (POOL, DB_NAME, USER, CONTEXT,
+    install_module)
+from trytond.transaction import Transaction
 
 
 class ToolsTestCase(unittest.TestCase):
@@ -118,13 +125,20 @@ class ToolsTestCase(unittest.TestCase):
                 ['OR', clause, ['AND', clause, ['OR', clause, clause]]]),
             (['OR', [clause]], ['OR', ['AND', clause]]),
             ([], []),
+            (['OR', clause, []], ['OR', clause, []]),
+            (['AND', clause, []], ['AND', clause, []]),
         )
         for i, j in tests:
             self.assertEqual(reduce_domain(i), j,
                     '%s -> %s != %s' % (i, reduce_domain(i), j))
 
+
 def suite():
-    return unittest.TestLoader().loadTestsFromTestCase(ToolsTestCase)
+    func = unittest.TestLoader().loadTestsFromTestCase
+    suite = unittest.TestSuite()
+    for testcase in (ToolsTestCase,):
+        suite.addTests(func(testcase))
+    return suite
 
 if __name__ == '__main__':
     suite = suite()
